import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.function.Consumer;

public class FormWarshipConfig extends JFrame {
    private JPanel mainPanel;
    private JPanel groupBoxConfig;
    private JLabel labelSpeed;
    private JLabel labelWeight;
    private JCheckBox checkBoxBodyKit;
    private JPanel groupBoxColors;
    private JPanel redPanel;
    private JPanel greenPanel;
    private JPanel bluePanel;
    private JPanel whitePanel;
    private JPanel grayPanel;
    private JPanel blackPanel;
    private JPanel yellowPanel;
    private JPanel purplePanel;
    private JLabel labelSimpleObject;
    private JLabel labelModifiedObject;
    private JCheckBox checkBoxSuperEngine;
    private JSpinner speedSpinner;
    private JSpinner weightSpinner;
    private JLabel paramLabel;
    private JLabel colorLabel;
    private JSpinner blocksSpinner;
    private JLabel labelBlocks;
    private JLabel labelRectangleBlocks;
    private JLabel labelRoundRectangleBLocks;
    private JLabel labelRoundBlocks;
    private JPanel panelObject;
    private JPanel pictureBox;
    private JLabel labelBaseColor;
    private JLabel labelDopColor;
    private JLabel labelSetBlocks;
    private JCheckBox checkBoxCabin;
    private JButton buttonAdd;
    private JButton buttonCancel;
    Consumer<DrawingWarship> EventAddWarship;
    DrawingWarship _warship;
    BlockCount _blocks;

    public FormWarshipConfig(){
        InitializeComponent();
    }

    public void AddEvent(Consumer<DrawingWarship> ev) { EventAddWarship = ev; }

    private void InitializeComponent(){
        setTitle("Warship");
        setSize(1000, 450);
        getContentPane().add(mainPanel);

        labelSimpleObject.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        labelModifiedObject.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        labelBaseColor.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        labelDopColor.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        labelRectangleBlocks.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        labelRoundBlocks.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        labelRoundRectangleBLocks.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        labelSetBlocks.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        pictureBox.setBorder(BorderFactory.createLineBorder(Color.BLACK));

        speedSpinner.setModel(new SpinnerNumberModel(100, 100, 1000, 1));
        weightSpinner.setModel(new SpinnerNumberModel(100, 100, 1000, 1));
        blocksSpinner.setModel(new SpinnerNumberModel(2, 2, 6, 2));

        MouseAdapter drag = new MouseAdapter() {
            @Override
            public void mousePressed(MouseEvent e) {
                setCursor(new Cursor(Cursor.HAND_CURSOR));
            }
            @Override
            public void mouseReleased(MouseEvent e) {
                setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                Drop((JComponent) e.getSource());
            }
        };

        MouseAdapter defCursor = new MouseAdapter() {
            @Override
            public void mouseExited(MouseEvent e) {
                setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
            }
        };

        pictureBox.addMouseListener(defCursor);
        labelBaseColor.addMouseListener(defCursor);
        labelDopColor.addMouseListener(defCursor);
        labelSetBlocks.addMouseListener(defCursor);

        redPanel.addMouseListener(drag);
        greenPanel.addMouseListener(drag);
        bluePanel.addMouseListener(drag);
        yellowPanel.addMouseListener(drag);
        whitePanel.addMouseListener(drag);
        grayPanel.addMouseListener(drag);
        blackPanel.addMouseListener(drag);
        purplePanel.addMouseListener(drag);

        labelSimpleObject.addMouseListener(drag);
        labelModifiedObject.addMouseListener(drag);
        labelRectangleBlocks.addMouseListener(drag);
        labelRoundBlocks.addMouseListener(drag);
        labelRoundRectangleBLocks.addMouseListener(drag);

        buttonAdd.addActionListener(e -> {
            EventAddWarship.accept(_warship);
            dispose();
        });

        buttonCancel.addActionListener(e -> dispose());
    }

    public void Drop(JComponent droppedItem) {
        if (droppedItem == null) {
            return;
        }

        if (droppedItem instanceof JPanel panel) {
            if (_warship == null)
                return;

            //Установка основного цвета
            if (labelBaseColor.getMousePosition() != null) {
                _warship.SetColor(panel.getBackground());
            }
            //Установка дополнительного цвета
            if (labelDopColor.getMousePosition() != null && _warship instanceof DrawingAircraftCarrier aircraftCarrier) {
                aircraftCarrier.SetDopColor(panel.getBackground());
            }
        }
        if (droppedItem instanceof JLabel label && pictureBox.getMousePosition() != null) {
            int speed = (int)speedSpinner.getValue();
            int weight = (int)weightSpinner.getValue();
            int blocks = (int)blocksSpinner.getValue();
            boolean bodyKit = checkBoxBodyKit.isSelected();
            boolean cabin = checkBoxCabin.isSelected();
            boolean superEngine = checkBoxSuperEngine.isSelected();
            //Обычный объект
            if (label == labelSimpleObject) {
                try { pictureBox.remove(_warship); } catch (Exception ex) { }
                _warship = new DrawingWarship(speed, weight, Color.WHITE, blocks);
            }
            //Модифицированный объект
            else if (label == labelModifiedObject) {
                try { pictureBox.remove(_warship); } catch (Exception ex) { }
                _warship = new DrawingAircraftCarrier(speed, weight, Color.WHITE, Color.WHITE, blocks, bodyKit, cabin, superEngine);
            }

            if (_warship != null) {
                _warship.SetPosition(pictureBox.getWidth() - 200, pictureBox.getHeight() - 150, pictureBox.getWidth(), pictureBox.getHeight());
                pictureBox.add(_warship, BorderLayout.CENTER);
                revalidate();
            }

        } else if (droppedItem instanceof JLabel label && labelSetBlocks.getMousePosition() != null && _warship != null) {
            //Квадратные блоки
            if (label == labelRectangleBlocks) {
                _warship.SetBlocks((int)blocksSpinner.getValue(), new DrawingBlocks(_blocks));
            }
            //Круглые блоки
            else if (label == labelRoundBlocks) {
                _warship.SetBlocks((int)blocksSpinner.getValue(), new DrawingRoundBlocks(_blocks));
            }
            //Закругленные блоки
            else if (label == labelRoundRectangleBLocks) {
                _warship.SetBlocks((int)blocksSpinner.getValue(), new DrawingRoundRectangleBlocks(_blocks));
            }
        }
    }
}
